from unittest.mock import patch

import pytest

from gso.products import Iptrunk
from gso.products.product_blocks import PhyPortCapacity
from gso.products.product_blocks.iptrunk import IptrunkType
from gso.schemas.enums import ProductType
from gso.services.crm import get_customer_by_name
from gso.services.subscriptions import get_product_id_by_name
from gso.workflows.utils import customer_selector
from test.workflows import (
    assert_aborted,
    assert_complete,
    assert_suspended,
    extract_state,
    resume_workflow,
    run_workflow,
)


@pytest.fixture
def input_form_wizard_data(router_subscription_factory, faker):
    router_side_a = router_subscription_factory()
    router_side_b = router_subscription_factory()

    create_ip_trunk_step = {
        "tt_number": faker.pystr(),
        "customer": getattr(customer_selector(), get_customer_by_name("GÉANT")["id"]),
        "geant_s_sid": faker.pystr(),
        "iptrunk_type": IptrunkType.DARK_FIBER,
        "iptrunk_description": faker.sentence(),
        "iptrunk_speed": PhyPortCapacity.HUNDRED_GIGABIT_PER_SECOND,
        "iptrunk_minimum_links": 5,
    }
    create_ip_trunk_side_a_step = {
        "iptrunk_sideA_node_id": router_side_a,
        "iptrunk_sideA_ae_iface": faker.pystr(),
        "iptrunk_sideA_ae_geant_a_sid": faker.pystr(),
        "iptrunk_sideA_ae_members": [faker.pystr() for _ in range(5)],
        "iptrunk_sideA_ae_members_descriptions": [faker.sentence() for _ in range(5)],
    }

    create_ip_trunk_side_b_step = {
        "iptrunk_sideB_node_id": router_side_b,
        "iptrunk_sideB_ae_iface": faker.pystr(),
        "iptrunk_sideB_ae_geant_a_sid": faker.pystr(),
        "iptrunk_sideB_ae_members": [faker.pystr() for _ in range(5)],
        "iptrunk_sideB_ae_members_descriptions": [faker.sentence() for _ in range(5)],
    }

    return [create_ip_trunk_step, create_ip_trunk_side_a_step, create_ip_trunk_side_b_step]


def _user_accept_and_assert_suspended(process_stat, step_log, extra_data=None):
    extra_data = extra_data or {}
    result, step_log = resume_workflow(process_stat, step_log, extra_data)
    assert_suspended(result)

    return result, step_log


@pytest.mark.workflow
@patch("gso.workflows.iptrunk.create_iptrunk.provisioning_proxy.check_ip_trunk")
@patch("gso.workflows.iptrunk.create_iptrunk.provisioning_proxy.provision_ip_trunk")
@patch("gso.workflows.iptrunk.create_iptrunk.infoblox.allocate_v6_network")
@patch("gso.workflows.iptrunk.create_iptrunk.infoblox.allocate_v4_network")
def test_successful_iptrunk_creation_with_standard_lso_result(
    mock_allocate_v4_network,
    mock_allocate_v6_network,
    mock_provision_ip_trunk,
    mock_check_ip_trunk,
    responses,
    input_form_wizard_data,
    faker,
):
    mock_allocate_v4_network.return_value = faker.ipv4_network()
    mock_allocate_v6_network.return_value = faker.ipv6_network()
    product_id = get_product_id_by_name(ProductType.IP_TRUNK)
    initial_site_data = [{"product": product_id}, *input_form_wizard_data]
    result, process_stat, step_log = run_workflow("create_iptrunk", initial_site_data)
    assert_suspended(result)

    standard_lso_result = {
        "callback_result": {
            "status": "ok",
            "job_id": "random_job_id",
            "output": "parsed_output",
            "return_code": 0,
        },
        "confirm": "ACCEPTED",
    }
    for _ in range(5):
        result, step_log = _user_accept_and_assert_suspended(process_stat, step_log, standard_lso_result)
        result, step_log = _user_accept_and_assert_suspended(process_stat, step_log)

    result, step_log = _user_accept_and_assert_suspended(process_stat, step_log, standard_lso_result)
    result, step_log = resume_workflow(process_stat, step_log, {})
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Iptrunk.from_subscription(subscription_id)

    assert "active" == subscription.status
    assert subscription.description == f"IP trunk, geant_s_sid:{input_form_wizard_data[0]['geant_s_sid']}"

    assert mock_provision_ip_trunk.call_count == 4
    assert mock_check_ip_trunk.call_count == 2


@pytest.mark.workflow
@patch("gso.workflows.iptrunk.create_iptrunk.provisioning_proxy.check_ip_trunk")
@patch("gso.workflows.iptrunk.create_iptrunk.provisioning_proxy.provision_ip_trunk")
@patch("gso.workflows.iptrunk.create_iptrunk.infoblox.allocate_v6_network")
@patch("gso.workflows.iptrunk.create_iptrunk.infoblox.allocate_v4_network")
def test_iptrunk_creation_fails_when_lso_return_code_is_one(
    mock_allocate_v4_network,
    mock_allocate_v6_network,
    mock_provision_ip_trunk,
    mock_check_ip_trunk,
    responses,
    input_form_wizard_data,
    faker,
):
    mock_allocate_v4_network.return_value = faker.ipv4_network()
    mock_allocate_v6_network.return_value = faker.ipv6_network()
    product_id = get_product_id_by_name(ProductType.IP_TRUNK)

    initial_site_data = [{"product": product_id}, *input_form_wizard_data]
    result, process_stat, step_log = run_workflow("create_iptrunk", initial_site_data)
    assert_suspended(result)

    standard_lso_result = {
        "callback_result": {
            "status": "ok",
            "job_id": "random_job_id",
            "output": "parsed_output",
            "return_code": 1,
        },
        "confirm": "ACCEPTED",
    }

    attempts = 3
    for _ in range(0, attempts - 1):
        result, step_log = _user_accept_and_assert_suspended(process_stat, step_log, standard_lso_result)
        result, step_log = _user_accept_and_assert_suspended(process_stat, step_log)

    result, step_log = _user_accept_and_assert_suspended(process_stat, step_log, standard_lso_result)
    result, step_log = resume_workflow(process_stat, step_log, {})
    assert_aborted(result)

    assert mock_provision_ip_trunk.call_count == attempts
    assert mock_check_ip_trunk.call_count == 0
